Package: rdecision
Title: Decision Analytic Modelling in Health Economics
Version: 1.3.1
Authors@R: c(
  person(given="Andrew", family="Sims", 
         email = "andrew.sims@newcastle.ac.uk", 
         role = c("aut", "cre"),
         comment = c(ORCID="0000-0002-9553-7278")),
  person(given="Kim", family="Keltie", 
         email="kim.keltie@nhs.net", 
         role=c("aut"),
         comment = c(ORCID="0000-0001-5108-6279")),
  person(given="Paola", family="Cognigni",
         email="paola.cognigni@nhs.net",
         role=c("aut"),
         comment = c(ORCID="0000-0001-5418-3103")))
Description: Classes and functions for modelling health care interventions 
  using decision trees and semi-Markov models. Mechanisms are provided for 
  associating an uncertainty distribution with each source variable and for
  ensuring transparency of the mathematical relationships between variables.
  The package terminology follows Briggs "Decision Modelling for Health
  Economic Evaluation" (2006, ISBN:978-0-19-852662-9).
Depends: R (>= 4.0.0)
Imports: graphics, grDevices, grid, R6, rlang (>= 0.4.2), stats, withr
Suggests: DiagrammeR, DiagrammeRsvg, rsvg, knitr, rmarkdown, testthat
        (>= 3.0.0)
License: GPL-3
Language: en-GB
Encoding: UTF-8
URL: https://github.com/ajsims1704/rdecision
BugReports: https://github.com/ajsims1704/rdecision/issues
VignetteBuilder: knitr, rmarkdown, DiagrammeR
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-31 14:55:03 UTC; simsa
Author: Andrew Sims [aut, cre] (ORCID: <https://orcid.org/0000-0002-9553-7278>),
  Kim Keltie [aut] (ORCID: <https://orcid.org/0000-0001-5108-6279>),
  Paola Cognigni [aut] (ORCID: <https://orcid.org/0000-0001-5418-3103>)
Maintainer: Andrew Sims <andrew.sims@newcastle.ac.uk>
Repository: CRAN
Date/Publication: 2026-07-31 16:10:02 UTC
Built: R 4.6.1; ; 2026-07-31 17:59:36 UTC; unix
